/*******************************************************************************
* File Name: SDCard_SPIM.c
* Version 1.20
*
* Description:
*  This file provides the source code to the API for the SCB Component.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SDCard_SPIM_PVT.h"

#if(SDCard_SPIM_SCB_MODE_I2C_INC)
    #include "SDCard_SPIM_I2C_PVT.h"
#endif /* (SDCard_SPIM_SCB_MODE_I2C_INC) */

#if(SDCard_SPIM_SCB_MODE_EZI2C_INC)
    #include "SDCard_SPIM_EZI2C_PVT.h"
#endif /* (SDCard_SPIM_SCB_MODE_EZI2C_INC) */

#if(SDCard_SPIM_SCB_MODE_SPI_INC || SDCard_SPIM_SCB_MODE_UART_INC)
    #include "SDCard_SPIM_SPI_UART_PVT.h"
#endif /* (SDCard_SPIM_SCB_MODE_SPI_INC || SDCard_SPIM_SCB_MODE_UART_INC) */


/***************************************
*    Run Time Configuration Vars
***************************************/

/* Stores internal component configuration for unconfigured mode */
#if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Common config vars */
    uint8 SDCard_SPIM_scbMode = SDCard_SPIM_SCB_MODE_UNCONFIG;
    uint8 SDCard_SPIM_scbEnableWake;
    uint8 SDCard_SPIM_scbEnableIntr;

    /* I2C config vars */
    uint8 SDCard_SPIM_mode;
    uint8 SDCard_SPIM_acceptAddr;

    /* SPI/UART config vars */
    volatile uint8 * SDCard_SPIM_rxBuffer;
    uint8  SDCard_SPIM_rxDataBits;
    uint32 SDCard_SPIM_rxBufferSize;

    volatile uint8 * SDCard_SPIM_txBuffer;
    uint8  SDCard_SPIM_txDataBits;
    uint32 SDCard_SPIM_txBufferSize;

    /* EZI2C config vars */
    uint8 SDCard_SPIM_numberOfAddr;
    uint8 SDCard_SPIM_subAddrSize;
#endif /* (SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG) */


/***************************************
*     Common SCB Vars
***************************************/

uint8 SDCard_SPIM_initVar = 0u;

#if !defined (CY_REMOVE_SDCard_SPIM_CUSTOM_INTR_HANDLER)
    cyisraddress SDCard_SPIM_customIntrHandler = NULL;
#endif /* !defined (CY_REMOVE_SDCard_SPIM_CUSTOM_INTR_HANDLER) */


/***************************************
*    Private Function Prototypes
***************************************/

static void SDCard_SPIM_ScbEnableIntr(void);
static void SDCard_SPIM_ScbModeStop(void);


/*******************************************************************************
* Function Name: SDCard_SPIM_Init
********************************************************************************
*
* Summary:
*  Initializes the SCB component to operate in one of the selected configurations:
*  I2C, SPI, UART or EZ I2C.
*  When the configuration is set to “Unconfigured SCB”, this function does
*  not do any initialization. Use mode-specific initialization APIs instead:
*  SCB_I2CInit, SCB_SpiInit, SCB_UartInit or SCB_EzI2CInit.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SDCard_SPIM_Init(void)
{
#if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG)
    if(SDCard_SPIM_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SDCard_SPIM_initVar = 0u; /* Clear init var */
    }
    else
    {
        /* Initialization was done before call */
    }

#elif(SDCard_SPIM_SCB_MODE_I2C_CONST_CFG)
    SDCard_SPIM_I2CInit();

#elif(SDCard_SPIM_SCB_MODE_SPI_CONST_CFG)
    SDCard_SPIM_SpiInit();

#elif(SDCard_SPIM_SCB_MODE_UART_CONST_CFG)
    SDCard_SPIM_UartInit();

#elif(SDCard_SPIM_SCB_MODE_EZI2C_CONST_CFG)
    SDCard_SPIM_EzI2CInit();

#endif /* (SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SDCard_SPIM_Enable
********************************************************************************
*
* Summary:
*  Enables the SCB component operation.
*  The SCB configuration should be not changed when the component is enabled.
*  Any configuration changes should be made after disabling the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SDCard_SPIM_Enable(void)
{
#if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Enable SCB block, only if it is already configured */
    if(!SDCard_SPIM_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SDCard_SPIM_CTRL_REG |= SDCard_SPIM_CTRL_ENABLED;

        SDCard_SPIM_ScbEnableIntr();
    }
#else
    SDCard_SPIM_CTRL_REG |= SDCard_SPIM_CTRL_ENABLED;

    SDCard_SPIM_ScbEnableIntr();
#endif /* (SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SDCard_SPIM_Start
********************************************************************************
*
* Summary:
*  Invokes SCB_Init() and SCB_Enable().
*  After this function call, the component is enabled and ready for operation.
*  When configuration is set to “Unconfigured SCB”, the component must first be
*  initialized to operate in one of the following configurations: I2C, SPI, UART
*  or EZ I2C. Otherwise this function does not enable the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  SDCard_SPIM_initVar - used to check initial configuration, modified
*  on first function call.
*
*******************************************************************************/
void SDCard_SPIM_Start(void)
{
    if(0u == SDCard_SPIM_initVar)
    {
        SDCard_SPIM_Init();
        SDCard_SPIM_initVar = 1u; /* Component was initialized */
    }

    SDCard_SPIM_Enable();
}


/*******************************************************************************
* Function Name: SDCard_SPIM_Stop
********************************************************************************
*
* Summary:
*  Disables the SCB component and its interrupt.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SDCard_SPIM_Stop(void)
{
#if(SDCard_SPIM_SCB_IRQ_INTERNAL)
    SDCard_SPIM_DisableInt();
#endif /* (SDCard_SPIM_SCB_IRQ_INTERNAL) */

    SDCard_SPIM_CTRL_REG &= (uint32) ~SDCard_SPIM_CTRL_ENABLED;  /* Disable SCB block */

#if(SDCard_SPIM_SCB_IRQ_INTERNAL)
    SDCard_SPIM_ClearPendingInt();
#endif /* (SDCard_SPIM_SCB_IRQ_INTERNAL) */

    SDCard_SPIM_ScbModeStop(); /* Calls scbMode specific Stop function */
}


/*******************************************************************************
* Function Name: SDCard_SPIM_SetCustomInterruptHandler
********************************************************************************
*
* Summary:
*  Registers a function to be called by the internal interrupt handler.
*  First the function that is registered is called, then the internal interrupt
*  handler performs any operations such as software buffer management functions
*  before the interrupt returns.  It is the user's responsibility not to break the
*  software buffer operations. Only one custom handler is supported, which is
*  the function provided by the most recent call.
*  At initialization time no custom handler is registered.
*
* Parameters:
*  func: Pointer to the function to register.
*        The value NULL indicates to remove the current custom interrupt
*        handler.
*
* Return:
*  None
*
*******************************************************************************/
void SDCard_SPIM_SetCustomInterruptHandler(cyisraddress func)
{
#if !defined (CY_REMOVE_SDCard_SPIM_CUSTOM_INTR_HANDLER)
    SDCard_SPIM_customIntrHandler = func; /* Register interrupt handler */
#else
    if(NULL != func)
    {
        /* Suppress compiler warning */
    }
#endif /* !defined (CY_REMOVE_SDCard_SPIM_CUSTOM_INTR_HANDLER) */
}


/*******************************************************************************
* Function Name: SDCard_SPIM_ScbModeEnableIntr
********************************************************************************
*
* Summary:
*  Enables an interrupt for a specific mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void SDCard_SPIM_ScbEnableIntr(void)
{
#if(SDCard_SPIM_SCB_IRQ_INTERNAL)
    #if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG)
        /* Enable interrupt in the NVIC */
        if(0u != SDCard_SPIM_scbEnableIntr)
        {
            SDCard_SPIM_EnableInt();
        }
    #else
        SDCard_SPIM_EnableInt();

    #endif /* (SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG) */
#endif /* (SDCard_SPIM_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: SDCard_SPIM_ScbModeStop
********************************************************************************
*
* Summary:
*  Calls the Stop function for a specific operation mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void SDCard_SPIM_ScbModeStop(void)
{
#if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG)
    if(SDCard_SPIM_SCB_MODE_I2C_RUNTM_CFG)
    {
        SDCard_SPIM_I2CStop();
    }
    else if(SDCard_SPIM_SCB_MODE_EZI2C_RUNTM_CFG)
    {
        SDCard_SPIM_EzI2CStop();
    }
    else
    {
        /* Do nohing for other modes */
    }
#elif(SDCard_SPIM_SCB_MODE_I2C_CONST_CFG)
    SDCard_SPIM_I2CStop();

#elif(SDCard_SPIM_SCB_MODE_EZI2C_CONST_CFG)
    SDCard_SPIM_EzI2CStop();

#endif /* (SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG) */
}


#if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: SDCard_SPIM_SetPins
    ********************************************************************************
    *
    * Summary:
    *  Sets the pins settings accordingly to the selected operation mode.
    *  Only available in the Unconfigured operation mode. The mode specific
    *  initialization function calls it.
    *  Pins configuration is set by PSoC Creator when a specific mode of operation
    *  is selected in design time.
    *
    * Parameters:
    *  mode:      Mode of SCB operation.
    *  subMode:   Sub-mode of SCB operation. It is only required for SPI and UART
    *             modes.
    *  uartTxRx:  Direction for UART.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SDCard_SPIM_SetPins(uint32 mode, uint32 subMode, uint32 uartTxRx)
    {
        uint32 hsiomSel [SDCard_SPIM_SCB_PINS_NUMBER];
        uint32 pinsDm   [SDCard_SPIM_SCB_PINS_NUMBER];
        uint32 pinsInBuf = 0u;

        uint32 i;

        /* Set default HSIOM to GPIO and Drive Mode to Analog Hi-Z */
        for(i = 0u; i < SDCard_SPIM_SCB_PINS_NUMBER; i++)
        {
            hsiomSel[i]  = SDCard_SPIM_HSIOM_DEF_SEL;
            pinsDm[i]    = SDCard_SPIM_PIN_DM_ALG_HIZ;
        }

        if((SDCard_SPIM_SCB_MODE_I2C   == mode) ||
           (SDCard_SPIM_SCB_MODE_EZI2C == mode))
        {
            hsiomSel[SDCard_SPIM_MOSI_SCL_RX_PIN_INDEX] = SDCard_SPIM_HSIOM_I2C_SEL;
            hsiomSel[SDCard_SPIM_MISO_SDA_TX_PIN_INDEX] = SDCard_SPIM_HSIOM_I2C_SEL;

            pinsDm[SDCard_SPIM_MOSI_SCL_RX_PIN_INDEX] = SDCard_SPIM_PIN_DM_OD_LO;
            pinsDm[SDCard_SPIM_MISO_SDA_TX_PIN_INDEX] = SDCard_SPIM_PIN_DM_OD_LO;
        }
    #if(!SDCard_SPIM_CY_SCBIP_V1_I2C_ONLY)
        else if(SDCard_SPIM_SCB_MODE_SPI == mode)
        {
            hsiomSel[SDCard_SPIM_MOSI_SCL_RX_PIN_INDEX] = SDCard_SPIM_HSIOM_SPI_SEL;
            hsiomSel[SDCard_SPIM_MISO_SDA_TX_PIN_INDEX] = SDCard_SPIM_HSIOM_SPI_SEL;
            hsiomSel[SDCard_SPIM_SCLK_PIN_INDEX]        = SDCard_SPIM_HSIOM_SPI_SEL;

            if(SDCard_SPIM_SPI_SLAVE == subMode)
            {
                /* Slave */
                pinsDm[SDCard_SPIM_MOSI_SCL_RX_PIN_INDEX] = SDCard_SPIM_PIN_DM_DIG_HIZ;
                pinsDm[SDCard_SPIM_MISO_SDA_TX_PIN_INDEX] = SDCard_SPIM_PIN_DM_STRONG;
                pinsDm[SDCard_SPIM_SCLK_PIN_INDEX]        = SDCard_SPIM_PIN_DM_DIG_HIZ;

            #if(SDCard_SPIM_SS0_PIN)
                /* Only SS0 is valid choice for Slave */
                hsiomSel[SDCard_SPIM_SS0_PIN_INDEX] = SDCard_SPIM_HSIOM_SPI_SEL;
                pinsDm  [SDCard_SPIM_SS0_PIN_INDEX] = SDCard_SPIM_PIN_DM_DIG_HIZ;
            #endif /* (SDCard_SPIM_SS1_PIN) */

            #if(SDCard_SPIM_MISO_SDA_TX_PIN)
                /* Disable input buffer */
                 pinsInBuf |= SDCard_SPIM_MISO_SDA_TX_PIN_MASK;
            #endif /* (SDCard_SPIM_MISO_SDA_TX_PIN_PIN) */
            }
            else /* (Master) */
            {
                pinsDm[SDCard_SPIM_MOSI_SCL_RX_PIN_INDEX] = SDCard_SPIM_PIN_DM_STRONG;
                pinsDm[SDCard_SPIM_MISO_SDA_TX_PIN_INDEX] = SDCard_SPIM_PIN_DM_DIG_HIZ;
                pinsDm[SDCard_SPIM_SCLK_PIN_INDEX]        = SDCard_SPIM_PIN_DM_STRONG;

            #if(SDCard_SPIM_SS0_PIN)
                hsiomSel [SDCard_SPIM_SS0_PIN_INDEX] = SDCard_SPIM_HSIOM_SPI_SEL;
                pinsDm   [SDCard_SPIM_SS0_PIN_INDEX] = SDCard_SPIM_PIN_DM_STRONG;
                pinsInBuf                                |= SDCard_SPIM_SS0_PIN_MASK;
            #endif /* (SDCard_SPIM_SS0_PIN) */

            #if(SDCard_SPIM_SS1_PIN)
                hsiomSel [SDCard_SPIM_SS1_PIN_INDEX] = SDCard_SPIM_HSIOM_SPI_SEL;
                pinsDm   [SDCard_SPIM_SS1_PIN_INDEX] = SDCard_SPIM_PIN_DM_STRONG;
                pinsInBuf                                |= SDCard_SPIM_SS1_PIN_MASK;
            #endif /* (SDCard_SPIM_SS1_PIN) */

            #if(SDCard_SPIM_SS2_PIN)
                hsiomSel [SDCard_SPIM_SS2_PIN_INDEX] = SDCard_SPIM_HSIOM_SPI_SEL;
                pinsDm   [SDCard_SPIM_SS2_PIN_INDEX] = SDCard_SPIM_PIN_DM_STRONG;
                pinsInBuf                                |= SDCard_SPIM_SS2_PIN_MASK;
            #endif /* (SDCard_SPIM_SS2_PIN) */

            #if(SDCard_SPIM_SS3_PIN)
                hsiomSel [SDCard_SPIM_SS3_PIN_INDEX] = SDCard_SPIM_HSIOM_SPI_SEL;
                pinsDm   [SDCard_SPIM_SS3_PIN_INDEX] = SDCard_SPIM_PIN_DM_STRONG;
                pinsInBuf                                |= SDCard_SPIM_SS3_PIN_MASK;
            #endif /* (SDCard_SPIM_SS2_PIN) */

                /* Disable input buffers */
            #if(SDCard_SPIM_MOSI_SCL_RX_PIN)
                pinsInBuf |= SDCard_SPIM_MOSI_SCL_RX_PIN_MASK;
            #endif /* (SDCard_SPIM_MOSI_SCL_RX_PIN) */

             #if(SDCard_SPIM_MOSI_SCL_RX_WAKE_PIN)
                pinsInBuf |= SDCard_SPIM_MOSI_SCL_RX_WAKE_PIN_MASK;
            #endif /* (SDCard_SPIM_MOSI_SCL_RX_WAKE_PIN) */

            #if(SDCard_SPIM_SCLK_PIN)
                pinsInBuf |= SDCard_SPIM_SCLK_PIN_MASK;
            #endif /* (SDCard_SPIM_SCLK_PIN) */
            }
        }
        else /* UART */
        {
            if(SDCard_SPIM_UART_MODE_SMARTCARD == subMode)
            {
                /* SmartCard */
                hsiomSel[SDCard_SPIM_MISO_SDA_TX_PIN_INDEX] = SDCard_SPIM_HSIOM_UART_SEL;
                pinsDm  [SDCard_SPIM_MISO_SDA_TX_PIN_INDEX] = SDCard_SPIM_PIN_DM_OD_LO;
            }
            else /* Standard or IrDA */
            {
                if(0u != (SDCard_SPIM_UART_RX & uartTxRx))
                {
                    hsiomSel[SDCard_SPIM_MOSI_SCL_RX_PIN_INDEX] = SDCard_SPIM_HSIOM_UART_SEL;
                    pinsDm  [SDCard_SPIM_MOSI_SCL_RX_PIN_INDEX] = SDCard_SPIM_PIN_DM_DIG_HIZ;
                }

                if(0u != (SDCard_SPIM_UART_TX & uartTxRx))
                {
                    hsiomSel[SDCard_SPIM_MISO_SDA_TX_PIN_INDEX] = SDCard_SPIM_HSIOM_UART_SEL;
                    pinsDm  [SDCard_SPIM_MISO_SDA_TX_PIN_INDEX] = SDCard_SPIM_PIN_DM_STRONG;

                #if(SDCard_SPIM_MISO_SDA_TX_PIN)
                     pinsInBuf |= SDCard_SPIM_MISO_SDA_TX_PIN_MASK;
                #endif /* (SDCard_SPIM_MISO_SDA_TX_PIN_PIN) */
                }
            }
        }
    #endif /* (!SDCard_SPIM_CY_SCBIP_V1_I2C_ONLY) */

    /* Configure pins: set HSIOM, DM and InputBufEnable */
    /* Note: the DR register settigns do not effect the pin output if HSIOM is other than GPIO */

    #if(SDCard_SPIM_MOSI_SCL_RX_PIN)
        SDCard_SPIM_SET_HSIOM_SEL(SDCard_SPIM_MOSI_SCL_RX_HSIOM_REG,
                                       SDCard_SPIM_MOSI_SCL_RX_HSIOM_MASK,
                                       SDCard_SPIM_MOSI_SCL_RX_HSIOM_POS,
                                       hsiomSel[SDCard_SPIM_MOSI_SCL_RX_PIN_INDEX]);

        SDCard_SPIM_spi_mosi_i2c_scl_uart_rx_SetDriveMode((uint8) pinsDm[SDCard_SPIM_MOSI_SCL_RX_PIN_INDEX]);

        SDCard_SPIM_SET_INP_DIS(SDCard_SPIM_spi_mosi_i2c_scl_uart_rx_INP_DIS,
                                     SDCard_SPIM_spi_mosi_i2c_scl_uart_rx_MASK,
                                     (0u != (pinsInBuf & SDCard_SPIM_MOSI_SCL_RX_PIN_MASK)));
    #endif /* (SDCard_SPIM_MOSI_SCL_RX_PIN) */

    #if(SDCard_SPIM_MOSI_SCL_RX_WAKE_PIN)
        SDCard_SPIM_SET_HSIOM_SEL(SDCard_SPIM_MOSI_SCL_RX_WAKE_HSIOM_REG,
                                       SDCard_SPIM_MOSI_SCL_RX_WAKE_HSIOM_MASK,
                                       SDCard_SPIM_MOSI_SCL_RX_WAKE_HSIOM_POS,
                                       hsiomSel[SDCard_SPIM_MOSI_SCL_RX_WAKE_PIN_INDEX]);

        SDCard_SPIM_spi_mosi_i2c_scl_uart_rx_wake_SetDriveMode((uint8)
                                                               pinsDm[SDCard_SPIM_MOSI_SCL_RX_WAKE_PIN_INDEX]);

        SDCard_SPIM_SET_INP_DIS(SDCard_SPIM_spi_mosi_i2c_scl_uart_rx_wake_INP_DIS,
                                     SDCard_SPIM_spi_mosi_i2c_scl_uart_rx_wake_MASK,
                                     (0u != (pinsInBuf & SDCard_SPIM_MOSI_SCL_RX_WAKE_PIN_MASK)));

         /* Set interrupt on falling edge */
        SDCard_SPIM_SET_INCFG_TYPE(SDCard_SPIM_MOSI_SCL_RX_WAKE_INTCFG_REG,
                                        SDCard_SPIM_MOSI_SCL_RX_WAKE_INTCFG_TYPE_MASK,
                                        SDCard_SPIM_MOSI_SCL_RX_WAKE_INTCFG_TYPE_POS,
                                        SDCard_SPIM_INTCFG_TYPE_FALLING_EDGE);
    #endif /* (SDCard_SPIM_MOSI_SCL_RX_WAKE_PIN) */

    #if(SDCard_SPIM_MISO_SDA_TX_PIN)
        SDCard_SPIM_SET_HSIOM_SEL(SDCard_SPIM_MISO_SDA_TX_HSIOM_REG,
                                       SDCard_SPIM_MISO_SDA_TX_HSIOM_MASK,
                                       SDCard_SPIM_MISO_SDA_TX_HSIOM_POS,
                                       hsiomSel[SDCard_SPIM_MISO_SDA_TX_PIN_INDEX]);

        SDCard_SPIM_spi_miso_i2c_sda_uart_tx_SetDriveMode((uint8) pinsDm[SDCard_SPIM_MISO_SDA_TX_PIN_INDEX]);

        SDCard_SPIM_SET_INP_DIS(SDCard_SPIM_spi_miso_i2c_sda_uart_tx_INP_DIS,
                                     SDCard_SPIM_spi_miso_i2c_sda_uart_tx_MASK,
                                    (0u != (pinsInBuf & SDCard_SPIM_MISO_SDA_TX_PIN_MASK)));
    #endif /* (SDCard_SPIM_MOSI_SCL_RX_PIN) */

    #if(SDCard_SPIM_SCLK_PIN)
        SDCard_SPIM_SET_HSIOM_SEL(SDCard_SPIM_SCLK_HSIOM_REG, SDCard_SPIM_SCLK_HSIOM_MASK,
                                       SDCard_SPIM_SCLK_HSIOM_POS, hsiomSel[SDCard_SPIM_SCLK_PIN_INDEX]);

        SDCard_SPIM_spi_sclk_SetDriveMode((uint8) pinsDm[SDCard_SPIM_SCLK_PIN_INDEX]);

        SDCard_SPIM_SET_INP_DIS(SDCard_SPIM_spi_sclk_INP_DIS,
                             SDCard_SPIM_spi_sclk_MASK,
                            (0u != (pinsInBuf & SDCard_SPIM_SCLK_PIN_MASK)));
    #endif /* (SDCard_SPIM_SCLK_PIN) */

    #if(SDCard_SPIM_SS0_PIN)
        SDCard_SPIM_SET_HSIOM_SEL(SDCard_SPIM_SS0_HSIOM_REG, SDCard_SPIM_SS0_HSIOM_MASK,
                                       SDCard_SPIM_SS0_HSIOM_POS, hsiomSel[SDCard_SPIM_SS0_PIN_INDEX]);

        SDCard_SPIM_spi_ss0_SetDriveMode((uint8) pinsDm[SDCard_SPIM_SS0_PIN_INDEX]);

        SDCard_SPIM_SET_INP_DIS(SDCard_SPIM_spi_ss0_INP_DIS,
                                     SDCard_SPIM_spi_ss0_MASK,
                                     (0u != (pinsInBuf & SDCard_SPIM_SS0_PIN_MASK)));
    #endif /* (SDCard_SPIM_SS1_PIN) */

    #if(SDCard_SPIM_SS1_PIN)
        SDCard_SPIM_SET_HSIOM_SEL(SDCard_SPIM_SS1_HSIOM_REG, SDCard_SPIM_SS1_HSIOM_MASK,
                                       SDCard_SPIM_SS1_HSIOM_POS, hsiomSel[SDCard_SPIM_SS1_PIN_INDEX]);

        SDCard_SPIM_spi_ss1_SetDriveMode((uint8) pinsDm[SDCard_SPIM_SS1_PIN_INDEX]);

        SDCard_SPIM_SET_INP_DIS(SDCard_SPIM_spi_ss1_INP_DIS,
                                     SDCard_SPIM_spi_ss1_MASK,
                                     (0u != (pinsInBuf & SDCard_SPIM_SS1_PIN_MASK)));
    #endif /* (SDCard_SPIM_SS1_PIN) */

    #if(SDCard_SPIM_SS2_PIN)
        SDCard_SPIM_SET_HSIOM_SEL(SDCard_SPIM_SS2_HSIOM_REG, SDCard_SPIM_SS2_HSIOM_MASK,
                                       SDCard_SPIM_SS2_HSIOM_POS, hsiomSel[SDCard_SPIM_SS2_PIN_INDEX]);

        SDCard_SPIM_spi_ss2_SetDriveMode((uint8) pinsDm[SDCard_SPIM_SS2_PIN_INDEX]);

        SDCard_SPIM_SET_INP_DIS(SDCard_SPIM_spi_ss2_INP_DIS,
                                     SDCard_SPIM_spi_ss2_MASK,
                                     (0u != (pinsInBuf & SDCard_SPIM_SS2_PIN_MASK)));
    #endif /* (SDCard_SPIM_SS2_PIN) */

    #if(SDCard_SPIM_SS3_PIN)
        SDCard_SPIM_SET_HSIOM_SEL(SDCard_SPIM_SS3_HSIOM_REG,  SDCard_SPIM_SS3_HSIOM_MASK,
                                       SDCard_SPIM_SS3_HSIOM_POS, hsiomSel[SDCard_SPIM_SS3_PIN_INDEX]);

        SDCard_SPIM_spi_ss3_SetDriveMode((uint8) pinsDm[SDCard_SPIM_SS3_PIN_INDEX]);

        SDCard_SPIM_SET_INP_DIS(SDCard_SPIM_spi_ss3_INP_DIS,
                                     SDCard_SPIM_spi_ss3_MASK,
                                     (0u != (pinsInBuf & SDCard_SPIM_SS3_PIN_MASK)));
    #endif /* (SDCard_SPIM_SS3_PIN) */
    }

#endif /* (SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG) */


/* [] END OF FILE */
