/*******************************************************************************
* File Name: SDCard_SPIM_SPI.c
* Version 1.20
*
* Description:
*  This file provides the source code to the API for the SCB Component in
*  SPI mode.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SDCard_SPIM_PVT.h"
#include "SDCard_SPIM_SPI_UART_PVT.h"

#if(SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG)

    /***************************************
    *  Config Structure Initialization
    ***************************************/

    const SDCard_SPIM_SPI_INIT_STRUCT SDCard_SPIM_configSpi =
    {
        SDCard_SPIM_SPI_MODE,
        SDCard_SPIM_SPI_SUB_MODE,
        SDCard_SPIM_SPI_CLOCK_MODE,
        SDCard_SPIM_SPI_OVS_FACTOR,
        SDCard_SPIM_SPI_MEDIAN_FILTER_ENABLE,
        SDCard_SPIM_SPI_LATE_MISO_SAMPLE_ENABLE,
        SDCard_SPIM_SPI_WAKE_ENABLE,
        SDCard_SPIM_SPI_RX_DATA_BITS_NUM,
        SDCard_SPIM_SPI_TX_DATA_BITS_NUM,
        SDCard_SPIM_SPI_BITS_ORDER,
        SDCard_SPIM_SPI_TRANSFER_SEPARATION,
        0u,
        NULL,
        0u,
        NULL,
        (uint32) SDCard_SPIM_SCB_IRQ_INTERNAL,
        SDCard_SPIM_SPI_INTR_RX_MASK,
        SDCard_SPIM_SPI_RX_TRIGGER_LEVEL,
        SDCard_SPIM_SPI_INTR_TX_MASK,
        SDCard_SPIM_SPI_TX_TRIGGER_LEVEL
    };


    /*******************************************************************************
    * Function Name: SDCard_SPIM_SpiInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for the SPI operation.
    *
    * Parameters:
    *  config:  Pointer to a structure that contains the following ordered list of
    *           fields. These fields match the selections available in the
    *           customizer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SDCard_SPIM_SpiInit(const SDCard_SPIM_SPI_INIT_STRUCT *config)
    {
        if(NULL == config)
        {
            CYASSERT(0u != 0u); /* Halt execution due to bad function parameter */
        }
        else
        {
            /* Configure pins */
            SDCard_SPIM_SetPins(SDCard_SPIM_SCB_MODE_SPI, config->mode, SDCard_SPIM_DUMMY_PARAM);

            /* Store internal configuration */
            SDCard_SPIM_scbMode       = (uint8) SDCard_SPIM_SCB_MODE_SPI;
            SDCard_SPIM_scbEnableWake = (uint8) config->enableWake;
            SDCard_SPIM_scbEnableIntr = (uint8) config->enableInterrupt;

            /* Set RX direction internal variables */
            SDCard_SPIM_rxBuffer      =         config->rxBuffer;
            SDCard_SPIM_rxDataBits    = (uint8) config->rxDataBits;
            SDCard_SPIM_rxBufferSize  = (uint8) config->rxBufferSize;

            /* Set TX direction internal variables */
            SDCard_SPIM_txBuffer      =         config->txBuffer;
            SDCard_SPIM_txDataBits    = (uint8) config->txDataBits;
            SDCard_SPIM_txBufferSize  = (uint8) config->txBufferSize;

            /* Configure SPI interface */
            SDCard_SPIM_CTRL_REG     = SDCard_SPIM_GET_CTRL_OVS(config->oversample)        |
                                            SDCard_SPIM_GET_CTRL_EC_AM_MODE(config->enableWake) |
                                            SDCard_SPIM_CTRL_SPI;

            SDCard_SPIM_SPI_CTRL_REG = SDCard_SPIM_GET_SPI_CTRL_CONTINUOUS    (config->transferSeperation)  |
                                            SDCard_SPIM_GET_SPI_CTRL_SELECT_PRECEDE(config->submode &
                                                                          SDCard_SPIM_SPI_MODE_TI_PRECEDES_MASK) |
                                            SDCard_SPIM_GET_SPI_CTRL_SCLK_MODE     (config->sclkMode)            |
                                            SDCard_SPIM_GET_SPI_CTRL_LATE_MISO_SAMPLE(config->enableLateSampling)|
                                            SDCard_SPIM_GET_SPI_CTRL_SUB_MODE      (config->submode)             |
                                            SDCard_SPIM_GET_SPI_CTRL_MASTER_MODE   (config->mode);

            /* Configure RX direction */
            SDCard_SPIM_RX_CTRL_REG     =  SDCard_SPIM_GET_RX_CTRL_DATA_WIDTH(config->rxDataBits)         |
                                                SDCard_SPIM_GET_RX_CTRL_BIT_ORDER (config->bitOrder)           |
                                                SDCard_SPIM_GET_RX_CTRL_MEDIAN    (config->enableMedianFilter) |
                                                SDCard_SPIM_SPI_RX_CTRL;

            SDCard_SPIM_RX_FIFO_CTRL_REG = SDCard_SPIM_GET_RX_FIFO_CTRL_TRIGGER_LEVEL(config->rxTriggerLevel);

            /* Configure TX direction */
            SDCard_SPIM_TX_CTRL_REG      = SDCard_SPIM_GET_TX_CTRL_DATA_WIDTH(config->txDataBits) |
                                                SDCard_SPIM_GET_TX_CTRL_BIT_ORDER (config->bitOrder)   |
                                                SDCard_SPIM_SPI_TX_CTRL;

            SDCard_SPIM_TX_FIFO_CTRL_REG = SDCard_SPIM_GET_TX_FIFO_CTRL_TRIGGER_LEVEL(config->txTriggerLevel);

            /* Configure interrupt with SPI handler but do not enable it */
            CyIntDisable    (SDCard_SPIM_ISR_NUMBER);
            CyIntSetPriority(SDCard_SPIM_ISR_NUMBER, SDCard_SPIM_ISR_PRIORITY);
            (void) CyIntSetVector(SDCard_SPIM_ISR_NUMBER, &SDCard_SPIM_SPI_UART_ISR);

            /* Configure interrupt sources */
            SDCard_SPIM_INTR_I2C_EC_MASK_REG = SDCard_SPIM_NO_INTR_SOURCES;
            SDCard_SPIM_INTR_SPI_EC_MASK_REG = SDCard_SPIM_NO_INTR_SOURCES;
            SDCard_SPIM_INTR_SLAVE_MASK_REG  = SDCard_SPIM_GET_SPI_INTR_SLAVE_MASK(config->rxInterruptMask);
            SDCard_SPIM_INTR_MASTER_MASK_REG = SDCard_SPIM_GET_SPI_INTR_MASTER_MASK(config->txInterruptMask);
            SDCard_SPIM_INTR_RX_MASK_REG     = SDCard_SPIM_GET_SPI_INTR_RX_MASK(config->rxInterruptMask);
            SDCard_SPIM_INTR_TX_MASK_REG     = SDCard_SPIM_GET_SPI_INTR_TX_MASK(config->txInterruptMask);

            /* Clear RX buffer indexes */
            SDCard_SPIM_rxBufferHead     = 0u;
            SDCard_SPIM_rxBufferTail     = 0u;
            SDCard_SPIM_rxBufferOverflow = 0u;

            /* Clrea TX buffer indexes */
            SDCard_SPIM_txBufferHead = 0u;
            SDCard_SPIM_txBufferTail = 0u;
        }
    }

#else

    /*******************************************************************************
    * Function Name: SDCard_SPIM_SpiInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for the SPI operation.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SDCard_SPIM_SpiInit(void)
    {
        /* Configure SPI interface */
        SDCard_SPIM_CTRL_REG     = SDCard_SPIM_SPI_DEFAULT_CTRL;
        SDCard_SPIM_SPI_CTRL_REG = SDCard_SPIM_SPI_DEFAULT_SPI_CTRL;

        /* Configure TX and RX direction */
        SDCard_SPIM_RX_CTRL_REG      = SDCard_SPIM_SPI_DEFAULT_RX_CTRL;
        SDCard_SPIM_RX_FIFO_CTRL_REG = SDCard_SPIM_SPI_DEFAULT_RX_FIFO_CTRL;

        /* Configure TX and RX direction */
        SDCard_SPIM_TX_CTRL_REG      = SDCard_SPIM_SPI_DEFAULT_TX_CTRL;
        SDCard_SPIM_TX_FIFO_CTRL_REG = SDCard_SPIM_SPI_DEFAULT_TX_FIFO_CTRL;

        /* Configure interrupt with SPI handler but do not enable it */
    #if(SDCard_SPIM_SCB_IRQ_INTERNAL)
            CyIntDisable    (SDCard_SPIM_ISR_NUMBER);
            CyIntSetPriority(SDCard_SPIM_ISR_NUMBER, SDCard_SPIM_ISR_PRIORITY);
            (void) CyIntSetVector(SDCard_SPIM_ISR_NUMBER, &SDCard_SPIM_SPI_UART_ISR);
    #endif /* (SDCard_SPIM_SCB_IRQ_INTERNAL) */

        /* Configure interrupt sources */
        SDCard_SPIM_INTR_I2C_EC_MASK_REG = SDCard_SPIM_SPI_DEFAULT_INTR_I2C_EC_MASK;
        SDCard_SPIM_INTR_SPI_EC_MASK_REG = SDCard_SPIM_SPI_DEFAULT_INTR_SPI_EC_MASK;
        SDCard_SPIM_INTR_SLAVE_MASK_REG  = SDCard_SPIM_SPI_DEFAULT_INTR_SLAVE_MASK;
        SDCard_SPIM_INTR_MASTER_MASK_REG = SDCard_SPIM_SPI_DEFAULT_INTR_MASTER_MASK;
        SDCard_SPIM_INTR_RX_MASK_REG     = SDCard_SPIM_SPI_DEFAULT_INTR_RX_MASK;
        SDCard_SPIM_INTR_TX_MASK_REG     = SDCard_SPIM_SPI_DEFAULT_INTR_TX_MASK;

    #if(SDCard_SPIM_INTERNAL_RX_SW_BUFFER_CONST)
        SDCard_SPIM_rxBufferHead     = 0u;
        SDCard_SPIM_rxBufferTail     = 0u;
        SDCard_SPIM_rxBufferOverflow = 0u;
    #endif /* (SDCard_SPIM_INTERNAL_RX_SW_BUFFER_CONST) */

    #if(SDCard_SPIM_INTERNAL_TX_SW_BUFFER_CONST)
        SDCard_SPIM_txBufferHead = 0u;
        SDCard_SPIM_txBufferTail = 0u;
    #endif /* (SDCard_SPIM_INTERNAL_TX_SW_BUFFER_CONST) */
    }
#endif /* (SDCard_SPIM_SCB_MODE_UNCONFIG_CONST_CFG) */


/*******************************************************************************
* Function Name: SDCard_SPIM_SetActiveSlaveSelect
********************************************************************************
*
* Summary:
*  Selects one of the four SPI slave select signals.
*  The component should be in one of the following states to change the active
*  slave select signal source correctly:
*   - the component is disabled
*   - the component has completed all transactions (TX FIFO is empty and the
*     SpiDone flag is set)
*  This function does not check that these conditions have been met.
*  At the initialization time the active slave select line is slave select 0.
*
* Parameters:
*  activeSelect: The four lines are available to utilize the Slave Select function.
*
* Return:
*  None
*
*******************************************************************************/
void SDCard_SPIM_SpiSetActiveSlaveSelect(uint32 activeSelect)
{
    uint32 spiCtrl;

    spiCtrl = SDCard_SPIM_SPI_CTRL_REG;

    spiCtrl &= (uint32) ~SDCard_SPIM_SPI_CTRL_SLAVE_SELECT_MASK;
    spiCtrl |= (uint32)  SDCard_SPIM_GET_SPI_CTRL_SS(activeSelect);

    SDCard_SPIM_SPI_CTRL_REG = spiCtrl;
}


#if(SDCard_SPIM_SPI_WAKE_ENABLE_CONST)
    /*******************************************************************************
    * Function Name: SDCard_SPIM_SpiSaveConfig
    ********************************************************************************
    *
    * Summary:
    *  Clears INTR_SPI_EC.WAKE_UP and enables it. This interrupt
    *  source triggers when the master assigns the SS line and wakes up the device.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SDCard_SPIM_SpiSaveConfig(void)
    {
        SDCard_SPIM_ClearSpiExtClkInterruptSource(SDCard_SPIM_INTR_SPI_EC_WAKE_UP);
        SDCard_SPIM_SetSpiExtClkInterruptMode(SDCard_SPIM_INTR_SPI_EC_WAKE_UP);
    }


    /*******************************************************************************
    * Function Name: SDCard_SPIM_SpiRestoreConfig
    ********************************************************************************
    *
    * Summary:
    *  Disables the INTR_SPI_EC.WAKE_UP interrupt source. After wakeup
    *  slave does not drive the MISO line and the master receives 0xFF.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SDCard_SPIM_SpiRestoreConfig(void)
    {
        SDCard_SPIM_SetSpiExtClkInterruptMode(SDCard_SPIM_NO_INTR_SOURCES);
    }
#endif /* (SDCard_SPIM_SPI_WAKE_ENABLE_CONST) */


/* [] END OF FILE */
