// Graphics library by ladyada/adafruit with init code from Rossum, speed adapted and converted to C by C.K.
// MIT license

#include "ST7735.h"

#define swap(a, b) { uint16_t t = a; a = b; b = t; }

#define ST7735_NOP 0x0
#define ST7735_SWRESET 0x01
#define ST7735_RDDID 0x04
#define ST7735_RDDST 0x09

#define ST7735_SLPIN  0x10
#define ST7735_SLPOUT  0x11
#define ST7735_PTLON  0x12
#define ST7735_NORON  0x13

#define ST7735_INVOFF 0x20
#define ST7735_INVON 0x21
#define ST7735_DISPOFF 0x28
#define ST7735_DISPON 0x29
#define ST7735_CASET 0x2A
#define ST7735_RASET 0x2B
#define ST7735_RAMWR 0x2C
#define ST7735_RAMRD 0x2E

#define ST7735_COLMOD 0x3A
#define ST7735_MADCTL 0x36


#define ST7735_FRMCTR1 0xB1
#define ST7735_FRMCTR2 0xB2
#define ST7735_FRMCTR3 0xB3
#define ST7735_INVCTR 0xB4
#define ST7735_DISSET5 0xB6

#define ST7735_PWCTR1 0xC0
#define ST7735_PWCTR2 0xC1
#define ST7735_PWCTR3 0xC2
#define ST7735_PWCTR4 0xC3
#define ST7735_PWCTR5 0xC4
#define ST7735_VMCTR1 0xC5

#define ST7735_RDID1 0xDA
#define ST7735_RDID2 0xDB
#define ST7735_RDID3 0xDC
#define ST7735_RDID4 0xDD

#define ST7735_PWCTR6 0xFC

#define ST7735_GMCTRP1 0xE0
#define ST7735_GMCTRN1 0xE1

// Font Indices
#define FONT_LENGTH		    0
#define FONT_FIXED_WIDTH	2
#define FONT_HEIGHT		    3
#define FONT_FIRST_CHAR		4
#define FONT_CHAR_COUNT		5
#define FONT_WIDTH_TABLE	6


#define DELAY 0x80
const uint8_t InitCmd[] = { // Init for 7735R, part 1 (red or green tab)
    21, // 15 commands in list:
    ST7735_SWRESET, DELAY, // 1: Software reset, 0 args, w/delay
    50, // 50 ms delay
    ST7735_SLPOUT , DELAY, // 2: Out of sleep mode, 0 args, w/delay
    50, // 50 ms delay
    ST7735_FRMCTR1, 3 , // 3: Frame rate ctrl - normal mode, 3 args:
    0x01, 0x2C, 0x2D, // Rate = fosc/(1x2+40) * (LINE+2C+2D)
    ST7735_FRMCTR2, 3 , // 4: Frame rate control - idle mode, 3 args:
    0x01, 0x2C, 0x2D, // Rate = fosc/(1x2+40) * (LINE+2C+2D)
    ST7735_FRMCTR3, 6 , // 5: Frame rate ctrl - partial mode, 6 args:
    0x01, 0x2C, 0x2D, // Dot inversion mode
    0x01, 0x2C, 0x2D, // Line inversion mode
    ST7735_INVCTR , 1 , // 6: Display inversion ctrl, 1 arg, no delay:
    0x07, // No inversion
    ST7735_PWCTR1 , 3 , // 7: Power control, 3 args, no delay:
    0xA2,
    0x02, // -4.6V
    0x84, // AUTO mode
    ST7735_PWCTR2 , 1 , // 8: Power control, 1 arg, no delay:
    0xC5, // VGH25 = 2.4C VGSEL = -10 VGH = 3 * AVDD
    ST7735_PWCTR3 , 2 , // 9: Power control, 2 args, no delay:
    0x0A, // Opamp current small
    0x00, // Boost frequency
    ST7735_PWCTR4 , 2 , // 10: Power control, 2 args, no delay:
    0x8A, // BCLK/2, Opamp current small & Medium low
    0x2A,
    ST7735_PWCTR5 , 2 , // 11: Power control, 2 args, no delay:
    0x8A, 0xEE,
    ST7735_VMCTR1 , 1 , // 12: Power control, 1 arg, no delay:
    0x0E,
    ST7735_INVOFF , 0 , // 13: Don't invert display, no args, no delay
    ST7735_MADCTL , 1 , // 14: Memory access control (directions), 1 arg:
    0xC0, // row addr/col addr, bottom to top refresh (RGB)
    ST7735_COLMOD , 1 , // 15: set color mode, 1 arg, no delay:
    0x05,
    ST7735_CASET , 4 , // 1: Column addr set, 4 args, no delay:
    0x00, 0x00, // XSTART = 0
    0x00, 0x7F, // XEND = 127
    ST7735_RASET , 4 , // 2: Row addr set, 4 args, no delay:
    0x00, 0x00, // XSTART = 0
    0x00, 0x9F,
    ST7735_GMCTRP1, 16 , // 1: Magical unicorn dust, 16 args, no delay:
    0x02, 0x1c, 0x07, 0x12,
    0x37, 0x32, 0x29, 0x2d,
    0x29, 0x25, 0x2B, 0x39,
    0x00, 0x01, 0x03, 0x10,
    ST7735_GMCTRN1, 16 , // 2: Sparkles and rainbows, 16 args, no delay:
    0x03, 0x1d, 0x07, 0x06,
    0x2E, 0x2C, 0x29, 0x2D,
    0x2E, 0x2E, 0x37, 0x3F,
    0x00, 0x00, 0x02, 0x10,
    ST7735_NORON , DELAY, // 3: Normal display on, no args, w/delay
    10, // 10 ms delay
    ST7735_DISPON , DELAY, // 4: Main screen turn on, no args w/delay
    10  // 10 ms delay
}; 



  uint8_t _cs, _rs, _rst, _sid, _sclk;
  uint8_t csport, rsport, sidport, sclkport;
  uint8_t cspin, rspin, sidpin, sclkpin;

  uint8_t madctl;
  uint16_t _backgroundColor;

  uint8_t _width;
  uint8_t _height;

  // Arduino GLCD font
  Font_t  Font;

// SPI Interface
inline void writecommand(uint8_t cmd)
{ 
    D_C_Write(0);   
    CS_Write(0);
    SPIM_WriteTxData(cmd);
    CS_Write(1);
    //CyDelayUs(1); // fixme: Delay??
}  

inline void writedata(uint8_t Data)
{ 
    D_C_Write(1);
    CS_Write(0);
    SPIM_WriteTxData(Data); 
    CS_Write(1);
    //CyDelayUs(1); // FIXME Delay ??
}  


// Companion code to the above tables. Reads and issues
// a series of LCD commands stored in PROGMEM byte array.
void ST7735_commandList(uint8_t const *addr) {
    uint8_t numCommands, numArgs;
    uint16_t ms;

    numCommands = *(addr++); // Number of commands to follow
    while(numCommands--) { // For each command...
        writecommand(*(addr++)); // Read, issue command
        numArgs = *(addr++); // Number of args to follow
        ms = numArgs & DELAY; // If hibit set, delay follows args
        numArgs &= ~DELAY; // Mask out delay bit
        while(numArgs--) { // For each argument...
            writedata(*(addr++)); // Read, issue argument
        }
        if (ms) {
            ms = *(addr++); // Read post-command delay time (ms)
                CyDelay(ms); // Delay in ms
        }
    }
}

/*
ST7735::ST7735(const uint8_t cs, const uint8_t rs, const uint8_t sid, 
  const uint8_t sclk, const uint8_t rst) {
  _cs = cs;
  _rs = rs;
  _sid = sid;
  _sclk = sclk;
  _rst = rst;
  _backgroundColor=WHITE;
}

ST7735::ST7735(const uint8_t cs, const uint8_t rs, const uint8_t rst) {
  _cs = cs;
  _rs = rs;
  _sid = 0;
  _sclk = 0;
  _rst = rst;
  _backgroundColor=WHITE;
}
*/


void ST7735_initR(const uint8_t width, const uint8_t height) {
  // SPI Interface has to be setup elsewhere
  // Is Display Reset required?

  // init display
  ST7735_commandList(InitCmd);
  madctl = 0xC0;
  _width = width;
  _height = height;
}

  uint8_t ST7735_width(void) {return(_width);};
  uint8_t ST7735_height(void) {return(_height);};
  uint16_t ST7735_backgroundColor(void) { return (_backgroundColor); };

void ST7735_setAddrWindow(const uint8_t x0, const uint8_t y0, const uint8_t x1, const uint8_t y1)
{
  writecommand(ST7735_CASET);  // column addr set
  writedata(0x00);
  writedata(x0+0);   // XSTART 
  writedata(0x00);
  writedata(x1+0);   // XEND

  writecommand(ST7735_RASET);  // row addr set
  writedata(0x00);
  writedata(y0+0);    // YSTART
  writedata(0x00);
  writedata(y1+0);    // YEND

  writecommand(ST7735_RAMWR);  // write to RAM
}

void ST7735_pushColor(const uint16_t color) {
  writedata(color >> 8);
  writedata(color);
/*
  *portOutputRegister(rsport) |= rspin;
  *portOutputRegister(csport) &= ~ cspin;

  spiwrite(color >> 8);    
  spiwrite(color);   

  *portOutputRegister(csport) |= cspin;
*/
}

void ST7735_setBackgroundColor(const uint16_t color)
{
  _backgroundColor=color;
}

void ST7735_drawPixel(const uint8_t x, const uint8_t y,const uint16_t color) {

  if ((x >= _width) || (y >= _height)) return;

  ST7735_setAddrWindow(x,y,x+1,y+1);
  writedata(color >> 8);
  writedata(color);

/*
  // setup for data
  *portOutputRegister(rsport) |= rspin;
  *portOutputRegister(csport) &= ~ cspin;

  spiwrite(color >> 8);    
  spiwrite(color);   

  *portOutputRegister(csport) |= cspin;
*/
}

void ST7735_fillScreen(const uint16_t color) {

  ST7735_setAddrWindow(0, 0, _width-1, _height-1);

  writedata(color >> 8);
  writedata(color);
  D_C_Write(1);
  CS_Write(0);
  uint8_t x;
  uint8_t y;
  for (x=0; x < _width; x++) {
    for (y=0; y < _height; y++) {
      SPIM_WriteTxData(color >> 8); 
      SPIM_WriteTxData(color);  
    }
  }
  CS_Write(1);
  //D_C_Write(0);

/*
  // setup for data
  *portOutputRegister(rsport) |= rspin;
  *portOutputRegister(csport) &= ~ cspin;

  for (uint8_t x=0; x < _width; x++) {
    for (uint8_t y=0; y < _height; y++) {
      spiwrite(color >> 8);    
      spiwrite(color);    
    }
  }

  *portOutputRegister(csport) |= cspin;
*/
}

void ST7735_miscScreen(uint16_t color) {

  ST7735_setAddrWindow(0, 0, _width-1, _height-1);

  // setup for data
  D_C_Write(1);
  CS_Write(0);
  uint8_t x;
  uint8_t y;
  for (x=0; x < _width; x++) {
    for (y=0; y < _height; y++) {
      SPIM_WriteTxData(color >> 8);    
      SPIM_WriteTxData(color);
      color++;
    }
  }
  CS_Write(1);
/*
  // setup for data
  *portOutputRegister(rsport) |= rspin;
  *portOutputRegister(csport) &= ~ cspin;

  for (uint8_t x=0; x < _width; x++) {
    for (uint8_t y=0; y < _height; y++) {
      spiwrite(color >> 8);    
      spiwrite(color);
      color++;
    }
  }

  *portOutputRegister(csport) |= cspin;
*/
}


// fill a circle
void ST7735_fillCircle(const uint8_t x0, const uint8_t y0, const uint8_t r, const uint16_t color) {
  int16_t f = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x = 0;
  int16_t y = r;

  ST7735_drawVerticalLine(x0, y0-r, 2*r+1, color);

  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x;
  
    ST7735_drawVerticalLine(x0+x, y0-y, 2*y+1, color);
    ST7735_drawVerticalLine(x0-x, y0-y, 2*y+1, color);
    ST7735_drawVerticalLine(x0+y, y0-x, 2*x+1, color);
    ST7735_drawVerticalLine(x0-y, y0-x, 2*x+1, color);
  }
}

// draw a circle outline
void ST7735_drawCircle(const uint8_t x0, const uint8_t y0, const uint8_t r, 
			const uint16_t color) {
  int16_t f = 1 - r;
  int16_t ddF_x = 1;
  int16_t ddF_y = -2 * r;
  int16_t x = 0;
  int16_t y = r;

  ST7735_drawPixel(x0, y0+r, color);
  ST7735_drawPixel(x0, y0-r, color);
  ST7735_drawPixel(x0+r, y0, color);
  ST7735_drawPixel(x0-r, y0, color);

  while (x<y) {
    if (f >= 0) {
      y--;
      ddF_y += 2;
      f += ddF_y;
    }
    x++;
    ddF_x += 2;
    f += ddF_x;
  
    ST7735_drawPixel(x0 + x, y0 + y, color);
    ST7735_drawPixel(x0 - x, y0 + y, color);
    ST7735_drawPixel(x0 + x, y0 - y, color);
    ST7735_drawPixel(x0 - x, y0 - y, color);
    
    ST7735_drawPixel(x0 + y, y0 + x, color);
    ST7735_drawPixel(x0 - y, y0 + x, color);
    ST7735_drawPixel(x0 + y, y0 - x, color);
    ST7735_drawPixel(x0 - y, y0 - x, color);
  }
}

uint8_t ST7735_getRotation() {
  return madctl;
}

void ST7735_setRotation(const uint8_t m) {
  madctl = m;
  writecommand(ST7735_MADCTL);  // memory access control (directions)
  writedata(madctl);  // row address/col address, bottom to top refresh

  if ( (m==0x60) || (m==0xA0) || (m==0x20) || (m==0xE0) )  {
    _width=160;
    _height=128;
  }
}

// draw a rectangle
void ST7735_drawRect(const uint8_t x, const uint8_t y, const uint8_t w, const uint8_t h, 
		      const uint16_t color) {
  // smarter version
  ST7735_drawHorizontalLine(x, y, w, color);
  ST7735_drawHorizontalLine(x, y+h-1, w, color);
  ST7735_drawVerticalLine(x, y, h, color);
  ST7735_drawVerticalLine(x+w-1, y, h, color);
}

void ST7735_fillRect(uint8_t x, uint8_t y, const uint8_t w, const uint8_t h, 
		      const uint16_t color) {
  // smarter version

  ST7735_setAddrWindow(x, y, x+w-1, y+h-1);

  // setup for data
  D_C_Write(1);
  CS_Write(0);
  // display controller writes first in X and increments Y after window width
  for (y=0; y < h; y++) {
    for (x=0; x < w; x++) {
      SPIM_WriteTxData(color >> 8);    
      SPIM_WriteTxData(color);    
    }
  }
  CS_Write(1);
}

void ST7735_drawVerticalLine(const uint8_t x, const uint8_t y, uint8_t length, const uint16_t color)
{
  if (x >= _width) return;
  if (y+length >= _height) length = _height-y-1;

  ST7735_drawFastLine(x,y,length,color,1);
}

void ST7735_drawHorizontalLine(const uint8_t x, const uint8_t y, uint8_t length, const uint16_t color)
{
  if (y >= _height) return;
  if (x+length >= _width) length = _width-x-1;

  ST7735_drawFastLine(x,y,length,color,0);
}

void ST7735_drawFastLine(const uint8_t x, const uint8_t y, uint8_t length, 
			  const uint16_t color, const uint8_t rotflag)
{
  if (rotflag) {
    ST7735_setAddrWindow(x, y, x, y+length);
  } else {
    ST7735_setAddrWindow(x, y, x+length, y+1);
  }
  // setup for data
  D_C_Write(1);
  CS_Write(0);
  while (length--) {
    SPIM_WriteTxData(color >> 8);    
    SPIM_WriteTxData(color);    
  }
  CS_Write(1);
}


// bresenham's algorithm - thx wikpedia
void ST7735_drawLine(int16_t x0, int16_t y0, int16_t x1, int16_t y1, 
		      const uint16_t color) {
  uint16_t steep = abs(y1 - y0) > abs(x1 - x0);
  if (steep) {
    swap(x0, y0);
    swap(x1, y1);
  }

  if (x0 > x1) {
    swap(x0, x1);
    swap(y0, y1);
  }

  uint16_t dx, dy;
  dx = x1 - x0;
  dy = abs(y1 - y0);

  int16_t err = dx / 2;
  int16_t ystep;

  if (y0 < y1) {
    ystep = 1;
  } else {
    ystep = -1;}

  for (; x0<=x1; x0++) {
    if (steep) {
      ST7735_drawPixel(y0, x0, color);
    } else {
      ST7735_drawPixel(x0, y0, color);
    }
    err -= dy;
    if (err < 0) {
      y0 += ystep;
      err += dx;
    }
  }
}


/**
 * Draw a rectangle with rounder corners
 *
 * @param x the x coordinate of the upper left corner of the rectangle
 * @param y the y coordinate of the upper left corner of the rectangle
 * @param width width of the rectangle
 * @param height height of the rectangle
 * @param radius ????
 * @param 16-bit color default BLACK 
 *
 * Draws a rectangle the same as DrawRect() but with rounded corners.
 * Radius is a value from 1 to half the height or width of the rectangle.
 * (what does that really mean?????)
 * Which is it, or is it a the smaller of the two?
 * FIXME FIXME need more description here.
 *
 * @see DrawRect()
 */

void ST7735_drawRoundRect(const uint8_t x, const uint8_t y, const uint8_t width, const uint8_t height, 
                           const uint8_t radius, const uint16_t color) 
{
  int16_t tSwitch; 
  uint8_t x1 = 0, y1 = radius;
  tSwitch = 3 - 2 * radius;
	
  while (x1 <= y1) {
    ST7735_drawPixel(x+radius - x1, y+radius - y1, color);
    ST7735_drawPixel(x+radius - y1, y+radius - x1, color);

    ST7735_drawPixel(x+width-radius + x1, y+radius - y1, color);
    ST7735_drawPixel(x+width-radius + y1, y+radius - x1, color);
	    
    ST7735_drawPixel(x+width-radius + x1, y+height-radius + y1, color);
    ST7735_drawPixel(x+width-radius + y1, y+height-radius + x1, color);

    ST7735_drawPixel(x+radius - x1, y+height-radius + y1, color);
    ST7735_drawPixel(x+radius - y1, y+height-radius + x1, color);

    if (tSwitch < 0) {
    	tSwitch += (4 * x1 + 6);
    } else {
      tSwitch += (4 * (x1 - y1) + 10);
      y1--;
    }
    x1++;
  }
	  	
  ST7735_drawHorizontalLine(x+radius, y, width-(2*radius), color);		// top
  ST7735_drawHorizontalLine(x+radius, y+height, width-(2*radius), color);	// bottom
  ST7735_drawVerticalLine(x, y+radius, height-(2*radius), color);		// left
  ST7735_drawVerticalLine(x+width, y+radius, height-(2*radius), color);	// right
}

uint8_t ST7735_isFixedWidthFont(void)
{
  return( (*(Font+FONT_LENGTH) == 0) && (*(Font+FONT_LENGTH+1) == 0) );
}

void ST7735_selectFont(const Font_t font)
{
	Font = font;
}

uint8_t ST7735_getFontHeight(void)
{
  return( *(Font+FONT_HEIGHT) );
}

uint8_t ST7735_putChar(uint8_t x, const uint8_t y, uint8_t c, const uint16_t color)
// returns x position after printed character
// function is a little bit speed optimized, with AT328p 16MHz: 
// 0.61ms for 5x7 character
// 1.26ms for a 7x15 character 
// 3.55ms for a 10x24 character
{
  if (Font == 0)
    return x; // no font selected

  /*
  * check for special character processing
  */
  if (c < 0x20) {  // special char, do nothing
    return x;
  }
	   
  uint8_t fontWidth = 0;
  uint8_t FontHeight = *(Font+FONT_HEIGHT);
  uint8_t bytes = (FontHeight+7)/8; /* calculates FontHeight in rounded up bytes */

  uint8_t firstChar = *(Font+FONT_FIRST_CHAR);
  uint8_t charCount = *(Font+FONT_CHAR_COUNT);
	
  uint16_t index = 0;

  if (c < firstChar || c >= (firstChar+charCount)) {
    return x; // invalid char
  }
  c-= firstChar;

  if ( ST7735_isFixedWidthFont() ) {
    fontWidth = *(Font+FONT_FIXED_WIDTH); 
    index = c*bytes*fontWidth+FONT_WIDTH_TABLE;
  }
  else  {
    // variable width font, read width data, to get the index
    /*
     * Because there is no table for the offset of where the data
     * for each character glyph starts, run the table and add up all the
     * widths of all the characters prior to the character we
     * need to locate.
     */
    uint8_t i;
    for (i=0; i<c; i++) {  
      index += *(Font+FONT_WIDTH_TABLE+i);
    }
    /*
     * Calculate the offset of where the font data
     * for our character starts.
     * The index value from above has to be adjusted because
     * there is potentialy more than 1 byte per column in the glyph,
     * when the characgter is taller than 8 bits.
     * To account for this, index has to be multiplied
     * by the FontHeight in bytes because there is one byte of font
     * data for each vertical 8 pixels.
     * The index is then adjusted to skip over the font fontWidth data
     * and the font header information.
     */
     index = index*bytes+charCount+FONT_WIDTH_TABLE;
     /*
      * Finally, fetch the fontWidth of our character
      */
     fontWidth = *(Font+FONT_WIDTH_TABLE+c);
  }  // end else

  // if character does not fit to the screen, write it at the end (no line wrap)
  if (x + fontWidth >= _width)  {
    x=_width-fontWidth;
  }

  // last but not least, draw the character
  ST7735_setAddrWindow(x, y, x+fontWidth-1, y+FontHeight-1);

  // setup display for data transmission
  D_C_Write(1);
  CS_Write(0);

  /*
   * Need to write to Display Memory in X direction
   * Y automatically increments due to set window size
   * Because GLCD fonts are organized in Y direction, need some ugly 
   * Font Data accesses to map the Font to the display memory
   * GLCD has stored the font in the following way:
   * Each byte is one X-step and eight Y-Steps. First X/Y pixel is given by the LSB
   * After all Bytes of one X line (font width) the next Y+8 pixel are coming
   * if less than 8-bit are used in a byte the MSB's are not used (at least for size 7)
   */
  uint8_t i;
  for (i=0; i<bytes; i++)	/* each vertical byte */
  { 
    uint16_t page = i*fontWidth; // page must be 16 bit to prevent overflow		
    uint8_t bitpnt=0x01;    // pointer to the bit to be drawn
    uint8_t nr_of_pixel=8;  // initial value for a byte
    const uint8_t* tmpFontPnt=Font+index+page;

    /* is it last byte of multibyte tall font? or a very small font? */
    uint8_t lastByte=( FontHeight > 8 && FontHeight < (i+1)*8 ) || (FontHeight < 8);
    if ( lastByte )
    /* is it last byte of multibyte tall font? or a very small font? */
    {
      nr_of_pixel=FontHeight-i*8;
      if (FontHeight<8)  // Workaround: bug in font-data for small fonts...
         bitpnt <<= (i+1)*8-FontHeight-1;        // remove unused bits
       else
         bitpnt <<= (i+1)*8-FontHeight;        
    }
    uint8_t p;
    for (p=0; p<nr_of_pixel; p++)  {   // p increments in y direction
      //uint8_t yp = i*8+p;
      uint8_t j;
      for (j=0; j<fontWidth; j++)  {   // write in x-direction
         uint8_t data = *(tmpFontPnt+j);
         if (data & bitpnt)  {
           SPIM_WriteTxData(color >> 8);    
           SPIM_WriteTxData(color);    
           //drawPixel(x+j, y+yp, color);
         }
         else  {
           SPIM_WriteTxData(_backgroundColor >> 8);    
           SPIM_WriteTxData(_backgroundColor);    
           //drawPixel(x+j, y+yp, _backgroundColor);
         }
       }  // for j
       bitpnt <<= 1;
     }  // end  for p
  }  // end for i
  //digitalWrite(_cs, HIGH);
  CS_Write(1);
  return (x+fontWidth); // valid char, return new x position
}

uint8_t ST7735_putS(uint8_t x, const uint8_t y, const char *string, const uint16_t color)
{
  uint8_t first=1;
  uint8_t fontHeight = *(Font+FONT_HEIGHT);
  char * str=(char *)string;

  while(*str)  {
    if (!first)  {
      // space between characters
      ST7735_drawVerticalLine(x,y,fontHeight, _backgroundColor);
      x++;
    }
    x=ST7735_putChar(x, y, (uint8_t)*str, color);
    str++;
    first=0;
  }
  return(x);
}



/**
 * Returns the pixel width of a character
 *
 * @param c character to be sized
 *
 * @return The width in pixels of the given character
 * including any inter-character gap pixels following the character when the character is
 * rendered on the display.
 *
 * @note The font for the character is the most recently selected font.
 *
 * @see StringWidth()
 * @see StringWidth_P()
 */

uint8_t ST7735_charWidth(uint8_t c)
{
  uint8_t width = 0;
	
  if  ( ST7735_isFixedWidthFont() ) {
    width = *(Font+FONT_FIXED_WIDTH)+1;  // there is 1 pixel pad here
  } 
  else  { 
    // variable width font 
    uint8_t firstChar = *(Font+FONT_FIRST_CHAR);
    uint8_t charCount = *(Font+FONT_CHAR_COUNT);
	
    // read width data
    if (c >= firstChar && c < (firstChar+charCount)) {
      c -= firstChar;
      width = *(Font+FONT_WIDTH_TABLE+c)+1;
    }
  }	
  return width;
}

/**
 * Returns the pixel width of a string
 *
 * @param str pointer to string stored in RAM
 *
 * @return the width in pixels of the sum of all the characters in the
 * the string pointed to by str. 
 *
 * @see CharWidth()
 * @see StringWidth_P()
 */

uint16_t ST7735_stringWidth(const char* str)
{
  uint16_t width = 0;
	
  while(*str != 0) {
    width += ST7735_charWidth(*str++);
  }	
  return width;
}


/**
 * Returns the pixel width of a character
 *
		 * @param str String class string
 *
 * @return the width in pixels of the sum of all the characters in the
 * the string pointed to by str. 
 *
 * @see CharWidth()
 * @see StringWidth()
 */
/* commented out to save memory
uint16_t ST7735::stringWidth(const String &str)
{
  uint16_t width = 0;

  for (uint16_t i = 0; i < str.length(); i++)
  {
    width += this->charWidth(str[i]);
  }
	
  return width;
}
*/



// end of file
